#ifndef META_GRAPHICS_MESH_GUARDIAN 
#define META_GRAPHICS_MESH_GUARDIAN

#ifndef MGML_GUARDIAN
	#include "MGML.h"
#endif

#include "MGML_IMAGE.h"
#include "MGML_ERROR.h"
#include "MGML_MEMORY.h"
#include <d3d9.h> 

#include <map>

#include <lib3ds/file.h>
#include <lib3ds/camera.h>
#include <lib3ds/mesh.h>
#include <lib3ds/node.h>
#include <lib3ds/material.h>
#include <lib3ds/matrix.h>
#include <lib3ds/vector.h>
#include <lib3ds/light.h>


namespace IMAGES
{
	class IMAGE;
};


namespace MGML_GRAPHICS
{

template <int n_,class T> 
struct VERTEX;

template <int   n, 
		  class T, 
		  class INDEX> class TRIANGLE;

template <int   dim, 
		  class T,
		  class HAL> class MESH;

		  class D3D_HAL;
		  class OpenGL_HAL;
		  
template <class T> struct OTC;

//////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////      MESH       //////////////////////////////////////////// 
//////////////////////////////////////////////////////////////////////////////////////////////////

//      . 
template <int dim, class T> 
class BaseMeshStruct
{
public:

  BaseMeshStruct()
  {
    vert = 0;
    num_vert = 0;

    indices = 0;
    num_index = 0;

    binding_sphere.pos.M[0] = 0;
    binding_sphere.pos.M[2] = 0;
    binding_sphere.pos.M[3] = 0;
    binding_sphere.setRadius(0);

    binding_box.vmin = binding_box.vmax = binding_sphere.pos;
  }

	enum {m_sz = ( dim == 4 ? 4: dim + 1)}; //  
	typedef MGML_MATH::MATRIX<m_sz,m_sz,T>	Matrix;
	typedef IMAGES::IMAGE IMAGE;

	unsigned int num_vert;
	VERTEX<dim,T>* vert;  

  unsigned int num_index;
	unsigned int*	 indices; 

	MGML_MATH::SPHERE<dim,T> binding_sphere;
	MGML_MATH::BOX<dim,T>	 binding_box;

	
	std::vector< IMAGE > textures;
	MGML_MEMORY::simple_vector< Material > materials;

	BaseMeshStruct& operator=(const BaseMeshStruct& rhs);

	virtual void Draw() const {}
	virtual void DrawWireFrameMode() const {}

	virtual void LoadToGPU(){}
	virtual void DrawBuffers() const {}

	virtual void LoadFrom3DS(std::string){}

	inline const MGML_MATH::MATRIX<m_sz,m_sz,T>& getMatrix() const {return binding_box.matrix();}
  inline const MGML_MATH::MATRIX<m_sz,m_sz,T>& getMatrixInverse() const {return binding_box.matrix_inverse();}

protected:
  void SwitchTrianglesToTransformedVertices();
  void SetTriangles();
};




//        HAL
template <int dim, class T,class HAL>
class HAL_DEPENDENCE_MESH: protected BaseMeshStruct<dim,T>
{

};

// OpenGL_Types_Constants
//  OpenGL, .  
template <class T> struct OTC { enum {RET = 0}; };

template <> struct OTC<float> { enum {RET = GL_FLOAT}; };
template <> struct OTC<double> { enum {RET = GL_DOUBLE}; };
template <> struct OTC<int> { enum {RET = GL_INT}; };
template <> struct OTC<short> { enum {RET = GL_SHORT}; };
template <> struct OTC<unsigned int> { enum {RET = GL_INT}; };
template <> struct OTC<unsigned short> { enum {RET = GL_SHORT}; };



/// OpenGL MESH
template <int dim, class T>
class HAL_DEPENDENCE_MESH<dim,T,OpenGL_HAL>: protected BaseMeshStruct<dim,T>
{
protected:
	enum {gl_type = OTC<T>::RET,			   //  OpenGL . 
		  mem_stride  = sizeof(VERTEX<dim,T>), // ,     (, ) 
	};

	unsigned int m_distlayListId; //   ,   .
  typedef BaseMeshStruct<dim,T> Base;

public:

	void Draw() const;
	void DrawWireFrameMode() const;

  HAL_DEPENDENCE_MESH& operator=(const HAL_DEPENDENCE_MESH& rhs)
	{
		Base::operator=(rhs);

    m_distlayListId = rhs.m_distlayListId;
		return *this;
	}

	void LoadToGPU();
	void DrawBuffers() const
	{
		Matrix m_t = getMatrix();
		m_t.Transpose();

		glPushMatrix();
			glMultMatrixf(m_t.L);
			glCallList(m_distlayListId);
		glPopMatrix();
	}
};





// Direct3D_Format_Types_Constants
//  D3D, .  
template <class T> struct D3DFMTTC { enum {RET = 0}; };

template <> struct D3DFMTTC<int> { enum {RET = D3DFMT_INDEX32}; };
template <> struct D3DFMTTC<short> { enum {RET = D3DFMT_INDEX16}; };

template <> struct D3DFMTTC<unsigned int> { enum {RET = D3DFMT_INDEX32}; };
template <> struct D3DFMTTC<unsigned short> { enum {RET = D3DFMT_INDEX16}; };


template <int dim, class T>
class HAL_DEPENDENCE_MESH<dim,T,D3D_HAL>: protected BaseMeshStruct<dim,T>
{

	typedef BaseMeshStruct<dim,T>				Base;
    typedef HAL_DEPENDENCE_MESH<dim,T,D3D_HAL> This;
protected:
	LPDIRECT3DVERTEXBUFFER9 pVertexBuffer;     
	LPDIRECT3DINDEXBUFFER9  pIndexBuffer;
	LPDIRECT3DDEVICE9       pD3DDevice;

	bool buffers_have_been_created;

	virtual void CreateBuffers();

	struct D3DCommonVert3
	{
	 MGML_MATH::VECTOR<3,float> p;
	 MGML_MATH::VECTOR<3,float> n;
	 MGML_MATH::VECTOR<2,float> u;
	};

public:

	inline bool is_buffers_initialized() const
	 {return (pVertexBuffer && pIndexBuffer && pD3DDevice);} 
	inline const LPDIRECT3DDEVICE9 getDevice() const
	 {return pD3DDevice;}

    This& operator=(const This& rhs);

	virtual void Draw() {}
	virtual void DrawWireFrameMode() const {}
	virtual void setDevice(LPDIRECT3DDEVICE9 pD3DDevice2){pD3DDevice = pD3DDevice2; }
	
	virtual void LoadToGPU();
	virtual void DrawBuffers() const;

	HAL_DEPENDENCE_MESH()
	{
		pVertexBuffer = 0;
		pIndexBuffer  = 0;
		buffers_have_been_created = false;
	}

	virtual ~HAL_DEPENDENCE_MESH()
	{
     if(pVertexBuffer) 
		pVertexBuffer->Release(); 

     if(pIndexBuffer) 
		pIndexBuffer->Release(); 
	}
};

// \\  HAL  

template <int dim, class T,class HAL> 
class MESH: public HAL_DEPENDENCE_MESH<dim,T,HAL>,
			      public MGML_MATH::SSE_Aligned_Object
{
 typedef HAL_DEPENDENCE_MESH<dim,T,HAL> Base;

public:

	typedef MGML_MATH::BOX<dim,T> Box;

  MESH();

  MESH(const MESH& rhs) { *this = rhs; }
	virtual ~MESH();

	MESH& operator=(const MESH& rhs)
	{
		Base::operator=(rhs);
		return *this;
	}

	void resizeVertices(int N)
	{
		delete [] vert;
		vert = new VERTEX<dim,T> [N];
		num_vert = N;
	}

	void resizeIndices(int N)
	{
		delete [] indices;
		indices = new unsigned int [N];
		num_index = N;
	}

	inline const MGML_MATH::MATRIX<m_sz,m_sz,T>& getMatrix() const {return binding_box.matrix();}
  inline const MGML_MATH::MATRIX<m_sz,m_sz,T>& getMatrixInverse() const {return binding_box.matrix_inverse();}

	inline unsigned int  numVertices() const {return num_vert;}
	inline unsigned int  numIndices()  const {return num_index;}

	inline const VERTEX<dim,T>*   pVertices() const  { return vert; }
	inline const unsigned int *   pIndices() const  { return indices; }

	inline void  setVertexPosition(const MGML_MATH::VECTOR<dim,T>& v,int N) 
	{ 
		if(numVertices() < (size_t)N)
			throw Error("out of range! in MESH::setVertexPosition");

		vert[N].pos = v;
	}

	
	inline void  setVertex(const MGML_GRAPHICS::VERTEX<dim,T>& v,int N) 
	{ 
		if(numVertices() < (size_t)N)
			throw Error("out of range! in MESH::setVertex");

		vert[N] = v;
	}


	inline void  setIndex(unsigned int i,int N) 
	{
		if(numIndices() < N)
			throw Error("out of range! in MESH::setIndex");

		indices[N] = i;
	}

  
	inline const MGML_MATH::SPHERE<dim,T>* pBindingSphere() const {return &binding_sphere;}
	inline const MGML_MATH::SPHERE<dim,T>& getBindingSphere() const {return binding_sphere;}

	inline const MGML_MATH::BOX<dim,T>* pBindingBox() const {return &binding_box;}
	inline const MGML_MATH::BOX<dim,T>& getBindingBox() const {return binding_box;}

	inline const MGML_MATH::VECTOR<dim,T>* pPos() const {return &binding_sphere.pos; }
	inline const MGML_MATH::VECTOR<dim,T>& getPos()  const {return binding_sphere.pos; }

	void AssignTexture(std::string file_name,int n)
	{
		if((unsigned)n >= textures.size())
			throw Error("Out of Range in MESH::AssignTexture");
		textures[n].LoadBMP(file_name.c_str()); 
		textures[n].LoadToGPU();
	}

	inline const Material& getMaterial(int n) {return materials[n];}
	inline int   NumMaterials() {return materials.size(); }

	inline const IMAGE* pTexture(int n) const  {return &textures[n];}
	inline const IMAGE& getTexture(int n) const  {return textures[n];}

	virtual void setMatrix(const MGML_MATH::MATRIX<m_sz,m_sz,T>& m);
	virtual void multMatrix(const MGML_MATH::MATRIX<m_sz,m_sz,T>& m);
  virtual void ApplyMatrix(const MGML_MATH::MATRIX<m_sz,m_sz,T>&);

	virtual void Draw() const { this->HAL_DEPENDENCE_MESH<dim,T,HAL>::Draw(); }
	virtual void DrawWireFrameMode() const { this->HAL_DEPENDENCE_MESH<dim,T,HAL>::DrawWireFrameMode();  }


	virtual void LoadFrom3DS(std::string);
	virtual void outDebugInfo(std::ostream&);

	virtual void CalcBindingSphere();
	virtual void CalcBindingBox();

	virtual void CalcNormalsAsOnFlatShading();

protected:

	virtual void LoadMaterialsFromThis3DS(Lib3dsFile *file,std::map<std::string,int>*);
	virtual void resize(int nver,int ni);

};








template<class Mesh>
class COMPLEX_MESH
{
public:

};







};















// =================================================================================
//
// =================================================================================
template <int dim, class T>
void MGML_GRAPHICS::BaseMeshStruct<dim,T>::SwitchTrianglesToTransformedVertices()
{
	
}

// =================================================================================
//
// =================================================================================
template <int dim, class T>
void MGML_GRAPHICS::BaseMeshStruct<dim,T>::SetTriangles()
{
	
}



template <int dim, class T>
inline MGML_GRAPHICS::BaseMeshStruct<dim,T>&
MGML_GRAPHICS::BaseMeshStruct<dim,T>::operator=(const BaseMeshStruct& rhs)
{
	num_vert	 = rhs.num_vert;
	num_index	 = rhs.num_index;
	

	binding_sphere = rhs.binding_sphere;
	binding_box    = rhs.binding_box;

	if(rhs.vert)
	{
    delete [] vert;
		vert = new VERTEX<dim,T> [num_vert];
		memcpy(vert,rhs.vert,num_vert*sizeof(VERTEX<dim,T>));
	} else vert = 0; 


	if(rhs.indices)
	{
    delete [] indices;
		indices = new unsigned int [num_index];
		memcpy(indices,rhs.indices,sizeof(unsigned int)*num_index);
	}  else indices =0;


//	basis[0] = rhs.basis[0];
//	basis[1] = rhs.basis[1];
//	basis[2] = rhs.basis[2];
  
  materials.resize(rhs.materials.size());
	for(unsigned int i=0;i<rhs.materials.size();i++)
		materials[i] = rhs.materials[i];
	
  textures.resize(rhs.textures.size());
	for(unsigned int i=0;i<rhs.textures.size();i++)
		textures[i] = rhs.textures[i];

	return *this;
}



// =================================================================================
//
// =================================================================================
template <int dim, class T, class HAL>
inline MGML_GRAPHICS::MESH<dim,T,HAL>::MESH()
{
  vert	  = 0;
	indices = 0;

	num_vert = 0;
	num_index = 0;

	textures.resize(1);
//	materials.resize(1);
}
// =================================================================================
//
// =================================================================================
template <int dim, class T, class HAL>
inline MGML_GRAPHICS::MESH<dim,T,HAL>::~MESH()
{
    delete [] vert;
	delete [] indices;
}
// =================================================================================
//
// =================================================================================
template <int dim, class T, class HAL>
void MGML_GRAPHICS::MESH<dim,T,HAL>::setMatrix(const MGML_MATH::MATRIX<m_sz,m_sz,T>& m)
{
 binding_box.setMatrix(m);
}

// =================================================================================
//
// =================================================================================
template <int dim, class T, class HAL>
void MGML_GRAPHICS::MESH<dim,T,HAL>::multMatrix(const MGML_MATH::MATRIX<m_sz,m_sz,T>& m)
{
 binding_box.setMatrix(m*binding_box.matrix());
}

// =================================================================================
//
// =================================================================================
template <int dim, class T, class HAL>
void MGML_GRAPHICS::MESH<dim,T,HAL>::ApplyMatrix(const MGML_MATH::MATRIX<m_sz,m_sz,T>& m)
{
 for(unsigned int i=0;i<num_vert;i++)
 {
	 if(dim == 4)
	 {
		vert[i].pos[3] = 1.0f;
		vert[i].norm[3] = 0.0f;
		MGML_MATH::matrix4x4f_mult_vector4f_fast(m.L,vert[i].pos.M,vert[i].pos.M);
		MGML_MATH::matrix4x4f_mult_vector4f_fast(m.L,vert[i].norm.M,vert[i].norm.M);
	 }
	 else
	 {
		vert[i].pos	 = m*vert[i].pos;
		vert[i].norm = m*vert[i].norm;
	 }

	 vert[i].norm.Normalize();
 }
}
// =================================================================================
//
// =================================================================================
template <int dim, class T, class HAL>
void MGML_GRAPHICS::MESH<dim,T,HAL>::resize(int nver,int ni)
{
    delete [] vert;
	delete [] indices;

	num_vert     = nver;
    num_index    = ni;

	vert      = new VERTEX<dim,T>		  [num_vert];
    indices   = new unsigned int		  [num_index];
}



// =================================================================================
//
// =================================================================================
template <int dim, class T, class HAL>
void  MGML_GRAPHICS::MESH<dim,T,HAL>::LoadMaterialsFromThis3DS(Lib3dsFile *file,std::map<std::string,int>* mm)
{
	std::map<std::string,int>& mat_map = *mm;

	materials.resize(32);
	unsigned int counter = 0;
	for(Lib3dsMaterial* mat=file->materials;mat!=0;mat=mat->next)
	{		
			Material my_mat;

			float r = mat->ambient[0] + mat->diffuse[0] + mat->specular[0];
			float g = mat->ambient[1] + mat->diffuse[1] + mat->specular[1];
			float b = mat->ambient[2] + mat->diffuse[2] + mat->specular[2];
			if(r>=1.0f) r=1.0f;
			if(g>=1.0f) g=1.0f;
			if(b>=1.0f) b=1.0f;
			//my_mat.color.set(r,g,b,mat->transparency);
			
			float ka = mat->ambient[0] + mat->ambient[1] + mat->ambient[2];
			float kd = mat->diffuse[0] + mat->diffuse[1] + mat->diffuse[2];
			float ks = mat->specular[0] + mat->specular[1] + mat->specular[2];
			if(ka>=1.0f) ka=1.0f;
			if(kd>=1.0f) kd=1.0f;
			if(ks>=1.0f) ks=1.0f;
//			my_mat.ka = ka;
//			my_mat.kd = kd;
//			my_mat.ks = ks;

			my_mat.k_reflect = ks;
			my_mat.k_refract = mat->transparency;

			my_mat.n         = 1.0f; //refractive component
			my_mat.power     = mat->shin_strength;
			
			if(counter >= materials.size())
				materials.resize(2*materials.size());
			
			materials[counter]= my_mat;
			mat_map[mat->name] = counter;
			counter++;
	}

	materials.resize(counter);
}


// =================================================================================
//
// =================================================================================
template <int dim, class T, class HAL>
void  MGML_GRAPHICS::MESH<dim,T,HAL>::LoadFrom3DS(std::string file_name)
{
	////////// using lib3ds ///////////
	Lib3dsFile *file=0;
	file=lib3ds_file_load(file_name.c_str());

	if(!file)
		throw MGML_ERROR::File_Error("LIB3DS: can not load file " + file_name);

	/* No nodes?  Fabricate nodes to display all the meshes. */
	if( !file->nodes )
	{
		Lib3dsMesh *mesh;
		Lib3dsNode *node;

		for(mesh = file->meshes; mesh != NULL; mesh = mesh->next)
		{
			node = lib3ds_node_new_object();
			strcpy(node->name, mesh->name);
			node->parent_id = LIB3DS_NO_PARENT;
			lib3ds_file_insert_node(file, node);
		}
	}

	lib3ds_file_eval(file, 0.0f);

	/////////////////////////////////
	Lib3dsMesh *mesh = 0;
	//Lib3dsNode *node = file->nodes; //for (p=file->nodes; p!=0; p=p->next) 
	unsigned int N_vert = 0,N_faces = 0;

	for (Lib3dsNode *node=file->nodes; node!=0; node=node->next)
	{
		mesh = lib3ds_file_mesh_by_name(file, node->data.object.morph);
		if( mesh == NULL )
			mesh = lib3ds_file_mesh_by_name(file, node->name);

		if(mesh==0)
			continue;

		N_faces += mesh->faces;
	}
	N_vert = N_faces*3;

	unsigned int max = (((size_t)1 << 31) - 1);
	if(N_vert >= max || N_faces*3 >= max)
		throw MGML_ERROR::Out_Of_Range("Mesh::LoadFrom3DS Overflow!,too many vertices");
	resize(N_vert,N_faces*3);

	//make a map of materials to represent ,aterial in triangle only as index

	std::map<std::string,int> material_map;
	LoadMaterialsFromThis3DS(file,&material_map);

	unsigned int offset_v = 0,offset_f=0;
	for (Lib3dsNode *node=file->nodes; node!=0; node=node->next)
	{

		mesh = lib3ds_file_mesh_by_name(file, node->data.object.morph);
		if( mesh == NULL )
			mesh = lib3ds_file_mesh_by_name(file, node->name);
	
		if(mesh==0)
			continue;

		// take matrix only for rirst mesh
		if(node==file->nodes)
		{
			// get matrix
		//	MGML_MATH::MATRIX<dim,dim,T> my_mat;
		//	
		//	memcpy(my_mat.L,mesh->matrix,dim*dim*sizeof(T));
		//	my_mat.Transpose();
		//	binding_box.matrix() = my_mat;
		}
	

		// calc normals
		Lib3dsVector *normalL=(Lib3dsVector*)malloc(3*sizeof(Lib3dsVector)*mesh->faces);
		lib3ds_mesh_calculate_normals(mesh, normalL);

		// now load all data
		for(unsigned p=0; p<mesh->faces; ++p)
		{
			Lib3dsFace *f=&mesh->faceL[p];
			Lib3dsMaterial *mat=0;

			if (f->material[0])
				mat=lib3ds_file_material_by_name(file, f->material);

			for(int i=0;i<3;i++)
			{
				unsigned int index = 3*p+i;

				if(offset_v+index >= numVertices())
					throw MGML_ERROR::Out_Of_Range("Mesh::LoadFrom3DS,vert array");

				vert[offset_v+index].pos.x() = mesh->pointL[f->points[i]].pos[0];
				vert[offset_v+index].pos.y() = mesh->pointL[f->points[i]].pos[1];
				vert[offset_v+index].pos.z() = mesh->pointL[f->points[i]].pos[2];
		  
				vert[offset_v+index].norm.x()= normalL[index][0];
				vert[offset_v+index].norm.y()= normalL[index][1];
				vert[offset_v+index].norm.z()= normalL[index][2];

				if(dim == 4)
				{
					vert[offset_v+index].pos.M[3] = 1.0f;
					vert[offset_v+index].norm.M[3] = 1.0f;
				}
			
				if(mesh->texelL != 0)
				{
					vert[offset_v+index].t.x() = mesh->texelL[f->points[i]][1];
					vert[offset_v+index].t.y() = mesh->texelL[f->points[i]][0];
				}
				else
					vert[offset_v+index].t.set(0,0);
				
				// materials
				if(mat == 0)
					vert[offset_v+index].material_id = 0;
				else
					vert[offset_v+index].material_id = material_map[mat->name];

				if(offset_v+index >= numIndices())
					throw MGML_ERROR::Out_Of_Range("Mesh::LoadFrom3DS,index array");

				indices[offset_v+index] = offset_v+index;
			} // for(int i=0;i<3;i++)

		}// for(unsigned p=0; p<mesh->faces; ++p) 
		offset_v += 3*(mesh->faces);
		offset_f += mesh->faces;
	//	size_t* face_by_vert = new size_t[mesh->points];

/*		for(size_t p=0; p<mesh->faces; ++p)
		{
			Lib3dsFace *f=&mesh->faceL[p];
			Lib3dsMaterial *mat=0;
			if (f->material[0])
				mat=lib3ds_file_material_by_name(file, f->material);

			size_t index = p*3;
			indices[index+0] = f->points[0];
			indices[index+1] = f->points[1];
			indices[index+2] = f->points[2];

		//	face_by_vert[f->points[0]] = p;
		//	face_by_vert[f->points[1]] = p;
		//	face_by_vert[f->points[2]] = p;
		}

		for(size_t i=0; i<mesh->points; ++i )
		{
			vert[i].pos.x() = mesh->pointL[i].pos[0];
			vert[i].pos.y() = mesh->pointL[i].pos[1];
			vert[i].pos.z() = mesh->pointL[i].pos[2];

			size_t normal_index = face_by_vert[i];
		//	vert[i].norm.x()= normalL[normal_index+2][0];
		//	vert[i].norm.y()= normalL[normal_index+2][1];
		//	vert[i].norm.z()= normalL[normal_index+2][2];

			if(mesh->texelL!=0)
			{
				vert[i].t.x() = mesh->texelL[i][1];
				vert[i].t.y() = mesh->texelL[i][0];
			}
			else
				vert[i].t.set(0,0);
		}*/

		free(normalL);
	//	delete [] face_by_vert;

	//	for(size_t i = 0;i<N_faces*3;i+=3)
	//	{
	//		MGML_ERROR::err_stream << indices[i] << "\t";
	//		MGML_ERROR::err_stream << indices[i+1] << "\t";
	//		MGML_ERROR::err_stream << indices[i+2] << "\n";
	//	}

	}// for (Lib3dsNode *node=file->nodes; node!=0; node=node->next)


}

// =================================================================================
//
// =================================================================================
template <int dim, class T, class HAL>
void  MGML_GRAPHICS::MESH<dim,T,HAL>::CalcBindingSphere()
{
	if(vert == 0)
		return;

	CalcBindingBox();
	binding_sphere.pos = binding_box.center();

	float max_radius = 0;
	for(unsigned int i=0;i<num_vert;i++)
	{
		float len = (vert[i].pos - binding_sphere.pos).lenSquare();
		if(len > max_radius)
			max_radius = len;
	}

	max_radius = sqrtf(max_radius);
	max_radius += MGML_MATH::EPSILON_E5;

	binding_sphere.setRadius(max_radius);
}

// =================================================================================
//
// =================================================================================

template <int dim, class T, class HAL>
void  MGML_GRAPHICS::MESH<dim,T,HAL>::CalcBindingBox()
{	
	binding_box.vmin = vert[0].pos;
	binding_box.vmax = vert[0].pos;

	for(unsigned int i=1;i<num_vert;i++)
	{
		for(unsigned int j=0;j<dim;j++) 
		{
			if(vert[i].pos.M[j] < binding_box.vmin.M[j] )
				binding_box.vmin.M[j]  = vert[i].pos.M[j];

			if(vert[i].pos.M[j] > binding_box.vmax.M[j] )
				binding_box.vmax.M[j]  = vert[i].pos.M[j];
		}
	}
	
	
	
}



// =================================================================================
//
// =================================================================================
template <int dim, class T, class HAL>
void  MGML_GRAPHICS::MESH<dim,T,HAL>::CalcNormalsAsOnFlatShading()
{
	typedef MGML_MATH::VECTOR<dim,T> vec;
	for(unsigned int i=0;i<numIndices();i+=3)
	{
		
		vec A = vert[indices[i+0]].pos;
		vec B = vert[indices[i+1]].pos;
		vec C = vert[indices[i+2]].pos;
		vec norm = (A-B)->*(A-C);

		norm.Normalize();
		vert[indices[i+0]].norm = norm;
		vert[indices[i+1]].norm = norm;
		vert[indices[i+2]].norm = norm;
	}

}
// =================================================================================
//
// =================================================================================
template <int dim, class T, class HAL>
void  MGML_GRAPHICS::MESH<dim,T,HAL>::outDebugInfo(std::ostream& out)
{


}





//////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////   HAL_DEPENDENCE_MESH   ////////////////////////////////////// 
//////////////////////////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////  OpenGL   ////////////////////////////////////////////// 
//////////////////////////////////////////////////////////////////////////////////////////////////
// =================================================================================
//
// =================================================================================

template <int dim, class T>
void MGML_GRAPHICS::HAL_DEPENDENCE_MESH<dim,
										T,
										MGML_GRAPHICS::OpenGL_HAL>::LoadToGPU()
{
	
	typedef MGML_GRAPHICS::VERTEX<dim,T> Vertex;

	m_distlayListId = glGenLists(1);

	glNewList(m_distlayListId, GL_COMPILE);
	
	glBegin(GL_TRIANGLES);
	for	(uint i=0;i<num_index;i+=3)
	{
		const Vertex& A = vert[indices[i+0]];
		const Vertex& B = vert[indices[i+1]];
		const Vertex& C = vert[indices[i+2]];
		
		glNormal3fv(A.norm.M);
		glTexCoord2fv(A.t.M);
		glVertex3fv(A.pos.M);

		glNormal3fv(B.norm.M);
		glTexCoord2fv(B.t.M);
		glVertex3fv(B.pos.M);

		glNormal3fv(C.norm.M);
		glTexCoord2fv(C.t.M);
		glVertex3fv(C.pos.M);
	}
	glEnd();
	glEndList();
}

// =================================================================================
//
// =================================================================================
template <int dim, class T>
void MGML_GRAPHICS::HAL_DEPENDENCE_MESH<dim,
										T,
										MGML_GRAPHICS::OpenGL_HAL>::Draw() const
{

    glEnableClientState(GL_VERTEX_ARRAY); 
	glEnableClientState(GL_NORMAL_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

    glVertexPointer(3,gl_type,mem_stride,(float*)vert);
    glNormalPointer(gl_type,mem_stride,((float*)vert + dim));
    glTexCoordPointer(2,gl_type,mem_stride,((float*)vert + dim*2));
    
    Matrix m_t = getMatrix();
	m_t.Transpose();

    glPushMatrix();
	 glMultMatrixf(m_t.L);
     glDrawElements(GL_TRIANGLES,num_index,GL_UNSIGNED_INT,indices);
	glPopMatrix();

	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);
	glDisableClientState(GL_VERTEX_ARRAY);

}
// =================================================================================
//
// =================================================================================
template <int dim, class T>
void MGML_GRAPHICS::HAL_DEPENDENCE_MESH<dim,
										T,
										MGML_GRAPHICS::OpenGL_HAL>::DrawWireFrameMode() const
{

    glEnableClientState(GL_VERTEX_ARRAY); 
	glEnableClientState(GL_NORMAL_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

    glVertexPointer(3,gl_type,mem_stride,(float*)vert);
    glNormalPointer(gl_type,mem_stride,((float*)vert + dim));
    glTexCoordPointer(2,gl_type,mem_stride,((float*)vert + dim*2));
 
	Matrix m_t = getMatrix();
	m_t.Transpose();

	glPushMatrix();
	 glMultMatrixf(m_t.L);
     glDrawElements(GL_LINES,num_index,GL_UNSIGNED_INT,indices);
	glPopMatrix();

	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);
	glDisableClientState(GL_VERTEX_ARRAY);

}

//////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////  Direct3D   //////////////////////////////////////////// 
//////////////////////////////////////////////////////////////////////////////////////////////////
template <int dim, class T>
MGML_GRAPHICS::HAL_DEPENDENCE_MESH<dim,T,MGML_GRAPHICS::D3D_HAL>&
MGML_GRAPHICS::HAL_DEPENDENCE_MESH<dim,T,MGML_GRAPHICS::D3D_HAL>
::operator=(const HAL_DEPENDENCE_MESH<dim,T,MGML_GRAPHICS::D3D_HAL>& rhs)
{

	Base::operator=(rhs);
	
	pD3DDevice = rhs.getDevice();

	if( rhs.is_buffers_initialized() )
	{
		CreateBuffers();
		LoadToGPU();
	}

	return *this;
}


// =================================================================================
//
// =================================================================================
template <int dim, class T>
void MGML_GRAPHICS::HAL_DEPENDENCE_MESH<dim,
										T,
										MGML_GRAPHICS::D3D_HAL>::CreateBuffers()
{
 try
 {
    pD3DDevice->CreateVertexBuffer( num_vert * sizeof(VERTEX<dim,T>),
												0,(D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_TEX1),
												D3DPOOL_DEFAULT, &pVertexBuffer, NULL );
		
	pD3DDevice->CreateIndexBuffer( num_index * sizeof(UINT), 
				0,D3DFMT_INDEX32,
				D3DPOOL_DEFAULT,&pIndexBuffer, 
				NULL);
 }
 catch(...)
 { throw MGML_ERROR::Hal_Error("Can not create vertex or index buffer");}

	buffers_have_been_created = true;
}
// =================================================================================
//
// =================================================================================
template <int dim, class T>
void MGML_GRAPHICS::HAL_DEPENDENCE_MESH<dim,
										T,
										MGML_GRAPHICS::D3D_HAL>::LoadToGPU()
{
   if(!buffers_have_been_created) CreateBuffers();

		try
		{
			if(dim == 4)
			{
				D3DCommonVert3* verts;
				pVertexBuffer->Lock( 0,num_vert*sizeof(D3DCommonVert3), (void**)&verts, 0 ); 

				for(unsigned int i=0;i<num_vert;i++)
				{
					verts[i].p.x = vert[i].pos.x;
					verts[i].p.y = vert[i].pos.y;
					verts[i].p.z = vert[i].pos.z;
					verts[i].n.x = vert[i].n.x;
					verts[i].n.y = vert[i].n.y;
					verts[i].n.z = vert[i].n.z;
					verts[i].u.x = vert[i].n.x;
					verts[i].u.y = vert[i].n.y;
				}

				pVertexBuffer->Unlock(); 
			}
			else
			{
				void* pBV;
				pVertexBuffer->Lock( 0,num_vert*sizeof(VERTEX<dim,T>), (void**)&pBV, 0 ); 
				memcpy( pBV, vert, num_vert*sizeof(VERTEX<dim,T>) );
				pVertexBuffer->Unlock();
			}
     
			void* pBI;
			pIndexBuffer->Lock( 0, sizeof(UINT)*num_index , (void**)&pBI, 0 );
			//
			memcpy( pBI, indices, sizeof(UINT)*num_index );
			// 
			pIndexBuffer->Unlock();
		}
		catch(...)
			{	throw MGML_ERROR::Hal_Error("A problems with copy model to GPU Memory"); }
 

}
// =================================================================================
//
// =================================================================================
template <int dim, class T>
void MGML_GRAPHICS::HAL_DEPENDENCE_MESH<dim,
										T,
										MGML_GRAPHICS::D3D_HAL>::DrawBuffers() const
{
	pD3DDevice->SetStreamSource( 0, pVertexBuffer, 0, sizeof(D3DCommonVert3) );
    pD3DDevice->SetFVF( (D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_TEX1) );
    pD3DDevice->SetIndices(pIndexBuffer);
    pD3DDevice->SetTransform( D3DTS_WORLD,(D3DMATRIX*)getMatrix().L);
	pD3DDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, 0, num_vert, 0, num_triangle);

}








#endif