#include "FujimotoTraverse.h"


bool IntersectRay2fBox2f(const Ray2f& r, const AABB2f& box, float& tmin)
{
   	// i know - thats bad
		const vec2f* V = box.CVPointer();
		// \\ now we can use V

		tmin = ( V[(r.dir[0] < 0)][0] - r.pos[0]) / r.dir[0];
		float tmax = ( V[1-(r.dir[0] < 0)][0] - r.pos[0]) / r.dir[0];
		float tymin = ( V[(r.dir[1] < 0)][1] - r.pos[1]) / r.dir[1];
		float tymax = ( V[1-(r.dir[1] < 0)][1] - r.pos[1]) / r.dir[1];

		if ( (tmin > tymax) || (tymin > tmax) )
			return false;

		if (tymin > tmin)
			tmin = tymin;

		if (tymax < tmax)
			tmax = tymax;

		return (tmax > 0) && ( tmin <= tmax);
}

void NextVoxel(int x, int y, const AABB2f& box)
{
  vec2f boxSize = box.vmax - box.vmin;
  vec2f t, t2;
  t.x() = (float)x/(float)CELL_NUMBER;
  t.y() = (float)y/(float)CELL_NUMBER;

  t2.x() = (float)(x+1)/(float)CELL_NUMBER;
  t2.y() = (float)(y+1)/(float)CELL_NUMBER;

  AABB2f smallBox;
  smallBox.vmin.x() = box.vmin.x() + t.x()*boxSize.x();
  smallBox.vmin.y() = box.vmin.y() + t.y()*boxSize.y();

  smallBox.vmax.x() = box.vmin.x() + t2.x()*boxSize.x();
  smallBox.vmax.y() = box.vmin.y() + t2.y()*boxSize.y();

 
  glBegin(GL_QUADS);
    glVertex2f(smallBox.vmin.x(), smallBox.vmin.y());
    glVertex2f(smallBox.vmin.x(), smallBox.vmax.y());
    glVertex2f(smallBox.vmax.x(), smallBox.vmax.y());
    glVertex2f(smallBox.vmax.x(), smallBox.vmin.y());
  glEnd();

}

void FujimotoTraverse2D(const Ray2f& ray, const AABB2f& box)
{
  float t_min;

  if(!IntersectRay2fBox2f(ray, box, t_min))
    return;

  if(t_min < 0)
    t_min = 0;

  float startX = ray.pos.x() + t_min*ray.dir.x();
  float startY = ray.pos.y() + t_min*ray.dir.y();

  int x = (int)((startX - box.vmin.x())/(box.vmax.x() - box.vmin.x())*CELL_NUMBER);
  int y = (int)((startY - box.vmin.y())/(box.vmax.y() - box.vmin.y())*CELL_NUMBER);

  if(x == CELL_NUMBER) x--;
  if(y == CELL_NUMBER) y--;

  vec2f boxSize = box.vmax - box.vmin;

  float tVoxelX, tVoxelY; 
  int stepX, stepY;
  
  if(ray.dir.x() > 0)
  {
    tVoxelX  = (float)(x+1)/(float)CELL_NUMBER;
    stepX = 1;
  }
  else
  {
    tVoxelX  = (float)x/(float)CELL_NUMBER;
    stepX = -1;
  }

  if(ray.dir.y() > 0)
  {
    tVoxelY = (float)(y+1)/(float)CELL_NUMBER;
    stepY = 1;
  }
  else
  {
    tVoxelY = (float)y/(float)CELL_NUMBER;
    stepY = -1;
  }

  float voxelMaxX = box.vmin.x() + tVoxelX*boxSize.x();
  float voxelMaxY = box.vmin.y() + tVoxelY*boxSize.y();

  float tMaxX = t_min + (voxelMaxX - startX)/ray.dir.x();
  float tMaxY = t_min + (voxelMaxY - startY)/ray.dir.y();

  const float voxelSize = (box.vmax.x() - box.vmin.x())/CELL_NUMBER;
  const float tDeltaX = voxelSize/fabs(ray.dir.x());
  const float tDeltaY = voxelSize/fabs(ray.dir.y());
  //NextVoxel(x, y, box);

  while(x < CELL_NUMBER && x >= 0 && 
        y < CELL_NUMBER && y >= 0)
  {
    NextVoxel(x, y, box);
    
    if(tMaxX < tMaxY)
    {
      tMaxX += tDeltaX;
      x += stepX;
    }
    else
    {
      tMaxY += tDeltaY;
      y += stepY;
    }

  };
}
