#include "HGrid.h"


HierarchicalGrid2D::HierarchicalGrid2D()
{
  m_top = 0;
  nodes = new HGridNode[MAX_TREE_SIZE];
}



HierarchicalGrid2D::~HierarchicalGrid2D()
{
  delete [] nodes;
}


int HierarchicalGrid2D::Build(const AABB2f& in_box, int prevOffset, int currDeep)
{
  HGridNode* node = nodes + m_top;
  node->upOffset = prevOffset;
  node->m_box = in_box;

  int currOffset = m_top;
  m_top++;

  for(int i=0;i<CELL_NUMBER;i++)
  {
    for(int j=0;j<CELL_NUMBER;j++)
    {
      float choice = rand()%3;
      
      if(choice < 1 && currDeep < MAX_DEEP)
      {
        float tX = ((float)i/(float)CELL_NUMBER);
        float tY = ((float)j/(float)CELL_NUMBER);
        float coordX = node->m_box.vmin.x() +  tX*(node->m_box.vmax.x() - node->m_box.vmin.x());
        float coordY = node->m_box.vmin.y() +  tY*(node->m_box.vmax.y() - node->m_box.vmin.y());

        float t2X = ((float)(i+1)/(float)CELL_NUMBER);
        float t2Y = ((float)(j+1)/(float)CELL_NUMBER);
        float coord2X = node->m_box.vmin.x() +  t2X*(node->m_box.vmax.x() - node->m_box.vmin.x());
        float coord2Y = node->m_box.vmin.y() +  t2Y*(node->m_box.vmax.y() - node->m_box.vmin.y());

        AABB2f box;
        box.vmin.set(coordX, coordY);
        box.vmax.set(coord2X, coord2Y);

        GridCell cell;
        cell.leaf = 0;
        cell.leftOffset = Build(box,currOffset,currDeep+1);
      }
      else
      {
        GridCell cell;
        cell.leftOffset = -1;
        cell.leaf = 1;

        node->grid[i+CELL_NUMBER*j] = cell; 
      }

    }
  }
  
  return currOffset;

}


void HierarchicalGrid2D::Draw()
{
  for(int i=0;i<MAX_TREE_SIZE;i++)
  {
    HGridNode* node = nodes+i;
    if(!node->Empty())
      node->Draw();
    else
      break;
  }
}

const AABB2f& HierarchicalGrid2D::GetBox(int offset)
{
  if(offset < m_top)
    return nodes[offset].m_box;
  else
    throw Out_Of_Range("HGridNode::GetBox too big offset");
}


void HGridNode::Draw()
{
  glBegin(GL_LINES);
  for(int i=0;i<CELL_NUMBER+1;i++)
  {
    float t = ((float)i/(float)CELL_NUMBER);
    float coordX = this->m_box.vmin.x() +  t*(this->m_box.vmax.x() - this->m_box.vmin.x());
    float coordY = this->m_box.vmin.y() +  t*(this->m_box.vmax.y() - this->m_box.vmin.y());

    glVertex2f(coordX, this->m_box.vmin.y());
    glVertex2f(coordX, this->m_box.vmax.y());

    glVertex2f(this->m_box.vmin.x(), coordY);
    glVertex2f(this->m_box.vmax.x(), coordY);
  }
  glEnd();
}
