#include <math.h>

//#include <glut.h> 

#ifndef MGML_GUARDIAN
	#include "../CSL/MGML.h"
#endif

#include "Input.h"
#include "Fonts.h"
#include "HGrid.h"
#include "FujimotoTraverse.h"


using namespace MGML;
//using namespace RAYTR;
using namespace MGML_MATH;
using namespace std;

using MGML_MATH::Intersect;

const int WinWidth = 1027;
const int WinHeight = 768;

const int MIN_X = -10;
const int MIN_Y = -10;

const int MAX_X = 10;
const int MAX_Y = 10;

Input input;  

HierarchicalGrid2D* pGrid;
Ray2f ray;


///////////////////////////////////////////////////////////////////////////////////
////
void OnReshape(int w, int h)
{
	// set the drawable region of the window to the maximum
	// available window size
	glViewport(0,0,w,h);

	// set up the projection matrix
	glMatrixMode(GL_PROJECTION);

	// clear any previous transform and set to the identity matrix
	glLoadIdentity();

	// just use an orthographic projection
	glOrtho(-MIN_X,MAX_X,-MIN_Y,MAX_Y,-1,1);

	// go back to modelview matrix so we can move the objects about
	glMatrixMode(GL_MODELVIEW);
}

///////////////////////////////////////////////////////////////////////////////////
////
void Init()
{
	try
	{
    cout << sizeof(GridCell) << endl;

    ray.pos.x() = -10;
    ray.pos.y() = -10;

    pGrid = new HierarchicalGrid2D;

    AABB2f box(vec2f(-10,-10), vec2f(10,10));
    pGrid->Build(box,-2,1);

		BuildFont();
	}
	catch(const Error& e)
	{
		cout << "Error: " << typeid(e).name() << endl;
		cout << e.GetMsg() << endl;
		exit(-1);
	}
	catch(...)
	{
		cout << "Unexpected Exeption!" << endl;
		exit(-1);
	}
}

///////////////////////////////////////////////////////////////////////////////////
////
void ShutDown()
{
	try
	{
    delete pGrid;

		KillFont();
	}
	catch(const Error& e)
	{
		cout << "Error: " << typeid(e).name() << endl;
		cout << e.GetMsg() << endl;
		exit(-1);
	}
	catch(...)
	{
		cout << "Unexpected Exeption at Exit!" << endl;
		exit(-1);
	}
	exit(0);
}

///////////////////////////////////////////////////////////////////////////////////
////
void ProcessInput()
{
  // work with input
  if(input.exit_status) 
	  ShutDown();

  ray.pos.x() += input.cam_mov.x();
  ray.pos.y() -= input.cam_mov.z();
  
  float r = 1;
  float angle = input.cam_rot.y()/10;
  ray.dir.x() = cos(angle);
  ray.dir.y() = sin(angle);
  ray.dir.Normalize();
}









///////////////////////////////////////////////////////////////////////////////////
////
void Display()
{
	static GLuint OldTick;			//  

  static GLuint FramesCount;		//  
  static GLuint StartTick;		//  
	static float FPS = 60;          //    

	static unsigned int counter = 0;

	static float t_process = 1;
	static float delta_t_process = 1;
	static float omega = 0.5;

	try
	{
	
		ProcessInput();

	  t_process += (omega/FPS);
		delta_t_process = (omega/FPS);

    glClear(GL_COLOR_BUFFER_BIT);

    /*glBegin(GL_TRIANGLES);

		  glColor3f(1,0,0);
		  glVertex3f(-5,-5,0);

		  glColor3f(0,1,0);
		  glVertex3f(5,-5,0);

		  glColor3f(0,0,1);
		  glVertex3f(0,5,0);

	  glEnd();*/
  
    glColor3f(0,1,0);
    pGrid->Draw();

    glColor3f(1,1,1);
    FujimotoTraverse2D(ray,pGrid->GetBox(0));
    //NextVoxel(0,0,pGrid->GetBox(0));
		
    glColor3f(1,0,0);
    vec2f end = ray.pos + 40*ray.dir;
    glBegin(GL_LINES);
      glVertex2f(ray.pos.x(), ray.pos.y());
      glVertex2f(end.x(), end.y());
    glEnd();
    // draw text here

		glutSwapBuffers();


		input.reset();

	}
	catch(const Error& e)
	{
		cout << "Error: " << typeid(e).name() << endl;
		cout << e.GetMsg() << endl;
		exit(-1);
	}
	catch(...)
	{
		cout << "Unexpected Exeption!" << endl;
		exit(-1);
	}
	
	// -----------------------    FPS. -----------------------
	FramesCount++;
	if(FramesCount > 100)
	{
		FramesCount=0;
        StartTick = glutGet(GLUT_ELAPSED_TIME);
	}
    char Title[32];
	GLuint st_tick_get = glutGet(GLUT_ELAPSED_TIME);
	if (st_tick_get!=StartTick)
	{
		    FPS = float(FramesCount)*1000/(st_tick_get-StartTick);
			sprintf(Title, "FPS=%d",(int)FPS);
			glutSetWindowTitle(Title);
	}
	//\\ -----------------------    FPS. -----------------------
}

void Reshape(int w, int h)
{
	// set the drawable region of the window to the maximum
	// available window size
	glViewport(0,0,w,h);

	// set up the projection matrix
	glMatrixMode(GL_PROJECTION);

	// clear any previous transform and set to the identity matrix
	glLoadIdentity();

	// just use an orthographic projection
	glOrtho(-12,12,-12,12,-1,1);

	// go back to modelview matrix so we can move the objects about
	glMatrixMode(GL_MODELVIEW);
}

void Mouse(int button, int state, int x, int y) { input.Mouse(button,state,x,y); }
void MouseMotion(int x, int y) { input.MouseMotion(x,y); }
//void Reshape(int Width,int Height) {  }               //   
void Keyboard(unsigned char key,int x,int y){ input.Keyboard(key,x,y); }   //
void KeyboardSpecial(int key, int x, int y) { input.KeyboardSpecial(key,x,y);  }   //  
void Mouse(int button, int state, int x, int y); //  
void Idle() { glutPostRedisplay(); } 

int main(int argc, char** argv)
{

  glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH | GLUT_STENCIL);
	glutInitWindowSize(WinWidth,WinHeight);
	glutInitWindowPosition((glutGet(GLUT_SCREEN_WIDTH)-WinWidth)/2, 
		                   (glutGet(GLUT_SCREEN_HEIGHT)-WinHeight)/2);
	glutCreateWindow("G&M Lab Battle Castle");
	
	glutDisplayFunc(Display);
	glutKeyboardFunc(Keyboard);
	glutSpecialFunc(KeyboardSpecial);
	glutReshapeFunc(Reshape);
	glutMouseFunc(Mouse);
	glutMotionFunc(MouseMotion);
	glutIdleFunc(Idle);

	Init();
			
	glutMainLoop();

	ShutDown();
}


